@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 12341 2020-06-11 14:59:06Z Lavender $
+# $Id: __init__.py 12811 2021-03-24 12:13:54Z Casey $
 #
 # Copyright (c) 2015 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -11,9 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Author: Lavender $ (last)
-# $Date: 2020-06-11 22:59:06 +0800 (週四, 11 六月 2020) $
-# $Revision: 12341 $
+# $Author: Casey $ (last)
+# $Date: 2021-03-24 20:13:54 +0800 (週三, 24 三月 2021) $
+# $Revision: 12811 $
 
 from __future__ import print_function
 
@@ -22,10 +22,9 @@ import sys
 import warnings
 import importlib
 
-from .Patch import attachURLs, attachDebugURLs
 from .Util import update, populateEnvToSettings
 
-from Iuppiter.DjangoUtil import extendInstalledApps, DJANGO_VERSION, extendMiddlewareClasses
+from Iuppiter.DjangoUtil import extendInstalledApps, DJANGO_VERSION, extendMiddlewareClasses, patterns
 
 # Default SERVER_MODE values.
 DEVELOPMENT = 1
@@ -41,17 +40,17 @@ SERVER_MODES = {
 def createWSGIHandler(cloudLogging=False, settingsModule=None, siteDir=None):
     """
     Create WSGIHandler for django and automatically initialize path settings.
-             
+
     @param cloudLogging True if you want to record errors to cloud logging
-                        service.               
+                        service.
     @param settingsModule Django settings module.
     @param siteDir Specify SITE_DIR to add to sys.path and site package.
     @return WSGIHandler.
-    """       
-    if ('DJANGO_SETTINGS_MODULE' not in os.environ or (settingsModule and 
+    """
+    if ('DJANGO_SETTINGS_MODULE' not in os.environ or (settingsModule and
             os.environ['DJANGO_SETTINGS_MODULE'] != settingsModule)):
         os.environ['DJANGO_SETTINGS_MODULE'] = settingsModule
-        
+
     if siteDir:
         import site
 
@@ -60,14 +59,14 @@ def createWSGIHandler(cloudLogging=False, settingsModule=None, siteDir=None):
 
         if siteDirAbs not in sys.path:
             sys.path.insert(0, siteDirAbs)
-    
-    from django.core.wsgi import get_wsgi_application               
+
+    from django.core.wsgi import get_wsgi_application
     handler = get_wsgi_application()
-   
+
     if cloudLogging:
         from Iuno.cloud.logging.middleware import WSGIHandler as _WSGIHandler
         handler = _WSGIHandler(handler)
-    
+
     return handler
 
 from Iuppiter.Util import colored, cprint
@@ -75,7 +74,7 @@ from Iuppiter.Util import colored, cprint
 def require(k, settingsDict):
     """
     Prompt require message while doing attachSettings.
-    
+
     @param k Settings key.
     @param settingsDict Settings dict.
     """
@@ -89,7 +88,7 @@ def warnPackage(package):
     """
     Prompt warning message for unable import certain package
     while doing attachSettings.
-    
+
     @param package Package name.
     """
     warnings.warn("%s '%s'" % (
@@ -100,7 +99,7 @@ def warnWithout(k, settingsDict, defaults):
     """
     Prompt warning message for without certain key in settings
     while doing attachSettings.
-    
+
     @param k Settings key.
     @param settingsDict Settings dict.
     @param defaults Defaults dict to provide default value.
@@ -116,7 +115,7 @@ def warnSet(k, settingsDict, defaults):
     """
     Prompt warning message for without set certain key in settings
     while doing attachSettings.
-    
+
     @param k Settings key.
     @param settingsDict Settings dict.
     @param defaults Defaults dict to provide default value.
@@ -132,7 +131,7 @@ def warnValue(k, settingsDict, defaults, value, msg=''):
     """
     Prompt warning message for user overwritten certain key in settings
     while doing attachSettings.
-    
+
     @param k Settings key.
     @param settingsDict Settings dict.
     @param defaults Defaults dict to provide default value.
@@ -148,12 +147,12 @@ def warnValue(k, settingsDict, defaults, value, msg=''):
                     attrs=['bold'])))
 
 
-            
+
 def checkImportPackage(packageName, settingsDict):
     """
-    Check the package you want to import and 
+    Check the package you want to import and
     raise an ImportError if the package is not available.
-    
+
     @param packageName Package name.
     @param settingsDict Settings dict.
     """
@@ -162,22 +161,22 @@ def checkImportPackage(packageName, settingsDict):
     except ImportError:
         warnPackage(packageName)
         upperName = packageName.upper()
-        
+
         startWithName = upperName.split('.')[0] + '_' + \
                         upperName.split('.')[1] + '_'
-                        
+
         for k in settingsDict:
             if k.startswith(startWithName):
                 # Project use packageName settings but
                 # package is not available.
                 raise
         return False
-            
+
 def warnPackageDisable(packageName, settingsDict, defaults):
     """
-    Show warning messages if using packages settings but 
+    Show warning messages if using packages settings but
     the package is disabled.
-    
+
     @param packageName Package name.
     @param settingsDict Settings dict.
     @param defaults Defaults dict to provide default value.
@@ -186,11 +185,11 @@ def warnPackageDisable(packageName, settingsDict, defaults):
     startWithName = upperName.split('.')[0] + '_' + \
                     upperName.split('.')[1] + '_'
     packageEnable = startWithName + 'ENABLE'
-    
+
     if settingsDict[packageEnable]:
-        
+
         for k in settingsDict:
-            
+
             if k.startswith(startWithName):
                 # Project use PACKAGE_settings but
                 # PACKAGE_ENABLE = False
@@ -198,7 +197,7 @@ def warnPackageDisable(packageName, settingsDict, defaults):
                           defaults, True,
                           'because %s are set.' % k)
                 break
-    
+
 def attachSettings(settingsDict, warns=True, printDetail=True):
     """
     Attach default settings to settings.py.
@@ -208,7 +207,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     @param warns Whether to show warnings or not.
     @param printDetail Whether to print information or not.
     """
-    
+
     args = sys.argv
 
     # if the command is manage.py runserver, then print the SERVER_MODE
@@ -218,12 +217,12 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         else:
             warns = False
             printDetail = False
-            
+
     # if the command is not manage.py runserver(ex: cms_dumpdata), do NOT print SERVER_MODE
     else:
         warns = False
         printDetail = False
-    
+
     settingsPath = os.path.abspath(settingsDict['__file__'])
     projectPath = os.path.abspath(os.path.join(
         os.path.dirname(settingsPath), '..'))
@@ -236,30 +235,30 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         'SERVER_MODE': DEVELOPMENT,
     })
 
-    # warnWithout('SERVER_MODE', settingsDict, serverModeDefaults)    
-    reversedServerModes = {v: k for k, v in list(serverModeDefaults.items()) 
+    # warnWithout('SERVER_MODE', settingsDict, serverModeDefaults)
+    reversedServerModes = {v: k for k, v in list(serverModeDefaults.items())
                                 if k != 'SERVER_MODE'}
-                                
+
     update(settingsDict, serverModeDefaults)
-    
-    populateEnvToSettings(('SERVER_MODE',), settingsDict, 
+
+    populateEnvToSettings(('SERVER_MODE',), settingsDict,
         lambda k, v, s: s.update({
             k: serverModeDefaults[v.upper()],
         })
     )
-    
+
     if printDetail:
         print(colored('SERVER_MODE', 'yellow', attrs=['bold']), end=' ')
         print(colored(" = ", 'white', attrs=['bold']), end=' ')
-        print(colored(reversedServerModes[settingsDict['SERVER_MODE']], 
-                    'cyan', attrs=['bold']))    
+        print(colored(reversedServerModes[settingsDict['SERVER_MODE']],
+                    'cyan', attrs=['bold']))
     #=========================================================================
 
     #=========================================================================
     # Debugging.
     DEBUG = True if settingsDict['SERVER_MODE'] not in (
         settingsDict['STAGE'], settingsDict['PRODUCTION']) else False
-        
+
     debugDefaults = {
         'DEBUG': DEBUG,
     }
@@ -267,12 +266,12 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     update(settingsDict, debugDefaults)
 
     # If environment variable specified DEBUG, we use it ignore SERVER_MODE.
-    populateEnvToSettings(list(debugDefaults.keys()), settingsDict, 
+    populateEnvToSettings(list(debugDefaults.keys()), settingsDict,
         lambda k, v, s: s.update({
             k: {'true': True, 'false': False}[v.lower()],
         })
     )
-    # Update back to variable if user specified DEBUG. 
+    # Update back to variable if user specified DEBUG.
     DEBUG = settingsDict['DEBUG']
     #=========================================================================
 
@@ -301,17 +300,17 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         ('Bear', 'bear@nuwainfo.com'),
         ('Bear', 'bear@tracedig.com'),
     )
-    
+
     import socket
-    
+
     ALLOWED_HOSTS = [
-        'localhost', 
+        'localhost',
         '127.0.0.1',
-        '::1', 
+        '::1',
         # 1.8.16 required for stage server. See:
         # https://docs.djangoproject.com/en/1.10/releases/
         # 1.8.16/#dns-rebinding-vulnerability-when-debug-true
-        '.nuwainfo.com', 
+        '.nuwainfo.com',
         '.tracedig.com',
         '.%s' % '.'.join(socket.gethostname().split('.')[1:]),
     ]
@@ -334,15 +333,15 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         'MANAGERS': ADMINS,
         'ALLOWED_HOSTS': ALLOWED_HOSTS,
     }
-    
+
     require('SECRET_KEY', settingsDict)
 
     update(settingsDict, adminDefaults)
-    
+
     populateEnvToSettings((
         'ADMIN_EMAIL',
         'ALLOWED_HOST',
-    ), settingsDict, 
+    ), settingsDict,
         # Use lambda to emulate switch case.
         lambda k, v, s: {
             'ADMIN_EMAIL': lambda: s.update({
@@ -374,9 +373,9 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     if 'DATABASE_URL' in os.environ:
         import dj_database_url
         dd = dj_database_url.config(default=os.environ['DATABASE_URL'])
-        
+
         _get = lambda k: settingsDict.get('DATABASE_%s' % k, dd[k])
-       
+
         databaseDefaults = {
             'DATABASE_ENGINE': dd['ENGINE'],
             'DATABASE_NAME': dd['NAME'],
@@ -391,10 +390,10 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
                     'USER': _get('USER'),
                     'PASSWORD': _get('PASSWORD'),
                     'HOST': _get('HOST'),
-                    'PORT': _get('PORT'),        
+                    'PORT': _get('PORT'),
                 }
             },
-        }        
+        }
     else:
         if settingsDict['SERVER_MODE'] == settingsDict['DEVELOPMENT']:
             dd = {
@@ -416,8 +415,8 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
                 'DATABASE_USER': dbName,
                 'DATABASE_PASSWORD': '%s-RX-0' % settingsDict['APP_NAME'],
                 # Support docker linked database as service_db.
-                'DATABASE_HOST': ('service_db' 
-                    if 'SERVICE_DB_NAME' in os.environ 
+                'DATABASE_HOST': ('service_db'
+                    if 'SERVICE_DB_NAME' in os.environ
                     else 'localhost'),
                 'DATABASE_PORT': '',
                 'OPTIONS':{
@@ -444,9 +443,9 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         databaseDefaults.update(dd)
 
     update(settingsDict, databaseDefaults)
-    
+
     # Use DATABASE_URL's value as DATABASES settings.
-    populateEnvToSettings(('DATABASE_URL',), settingsDict, 
+    populateEnvToSettings(('DATABASE_URL',), settingsDict,
         lambda k, v, s: s.update(databaseDefaults))
     #=========================================================================
 
@@ -457,9 +456,9 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
 
     # https://docs.sentry.io/clients/python/integrations/django/
     # #integration-with-logging
-    # Please note the raven logging docs have bugs, you will get 
-    # ValueError: Unable to configure handler 'sentry': __init__() got an 
-    # unexpected keyword argument 'tags' 
+    # Please note the raven logging docs have bugs, you will get
+    # ValueError: Unable to configure handler 'sentry': __init__() got an
+    # unexpected keyword argument 'tags'
     # if you follow the docs.
 
     loggingDefaults = {
@@ -488,19 +487,19 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
                 # 'level': 'DEBUG',
                 # 'class': 'logging.FileHandler',
                 # 'filename': os.path.join(projectPath, 'debug.log'),
-            # },        
+            # },
         # })
         # loggingDefaults['LOGGING']['loggers'].update({
             # 'django': {
                 # 'handlers': ['debug_file'],
                 # 'level': 'DEBUG',
                 # 'propagate': True,
-            # },        
+            # },
         # })
 
 
     update(settingsDict, loggingDefaults)
-    
+
     populateEnvToSettings((
         'LOGGING',
     ), settingsDict)
@@ -511,9 +510,9 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     i18nDefaults = {
         # On Linux systems, a value of None will cause Django to use the same
         # timezone as the operating system.
-        # If running in a Windows environment this must be set to the same as 
+        # If running in a Windows environment this must be set to the same as
         # your system time zone.
-        'TIME_ZONE': ('Asia/Taipei' if (sys.platform.startswith('win') or 
+        'TIME_ZONE': ('Asia/Taipei' if (sys.platform.startswith('win') or
                                         sys.platform.startswith('darwin'))
                                     else None),
         'LANGUAGE_CODE': 'en-us',
@@ -529,14 +528,14 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             os.path.join(projectPath, 'locale'),
         ),
     }
-    
-    if (DJANGO_VERSION >= 11100 and 
-        (not sys.platform.startswith('win') or 
+
+    if (DJANGO_VERSION >= 11100 and
+        (not sys.platform.startswith('win') or
          not sys.platform.startswith('darwin'))): # django >= 1.11
         i18nDefaults['TIME_ZONE'] = 'UTC'
-    
+
     update(settingsDict, i18nDefaults)
-    
+
     populateEnvToSettings((
         'TIME_ZONE',
         'LANGUAGE_CODE',
@@ -546,8 +545,8 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     #=========================================================================
     # static files.
     staticURL = '/static/'
-    
-    if not DEBUG:   
+
+    if not DEBUG:
         staticDefaults = {
             'STATIC_ROOT': os.path.join(projectPath, 'static'),
             'STATIC_URL': staticURL,
@@ -557,10 +556,10 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
 
         if settingsDict['SERVER_MODE'] == settingsDict['DEVELOPMENT']:
             warnings.warn("%s '%s'" % (
-                colored("If you want to serve static_files", 'yellow', 
+                colored("If you want to serve static_files", 'yellow',
                         attrs=['bold']),
                 colored("Please run server with arg --insecure under the "
-                        "development mode if DEBUG=False.", 'cyan', 
+                        "development mode if DEBUG=False.", 'cyan',
                         attrs=['bold'])))
     else:
         staticDefaults = {
@@ -571,13 +570,13 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             'MEDIA_ROOT' : os.path.join(projectPath, 'media'),
             'MEDIA_URL' : '/media/',
         }
-        
+
     update(settingsDict, staticDefaults)
-    
+
     populateEnvToSettings((
         'STATIC_URL',
         'MEDIA_URL',
-    ), settingsDict)    
+    ), settingsDict)
     #=========================================================================
 
     #=========================================================================
@@ -591,9 +590,9 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         'EMAIL_HOST_PASSWORD': "test25025529",
         'EMAIL_BACKEND': 'Iuno.Email.ConfiguredEmailBackend',
     }
-    
+
     update(settingsDict, emailDefaults)
-    
+
     populateEnvToSettings(list(emailDefaults.keys()), settingsDict, evalKeys=(
         'EMAIL_PORT',
         'EMAIL_USE_TLS',
@@ -608,9 +607,9 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
 
     update(settingsDict, rootUrlDefaults)
     #=========================================================================
-    
+
     #=========================================================================
-    # HTTPS and HTTP related options.    
+    # HTTPS and HTTP related options.
     if 'SECURE_PROXY_SSL_HEADER' not in settingsDict:
         https = os.environ.get('HTTPS', None)
         if https == 'on':
@@ -622,7 +621,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         # Allow userTrack uses iframe to track user motion.
         'X_FRAME_OPTIONS': "ALLOW-FROM https://session.nuwainfo.com/"
     }
-    update(settingsDict, defaultHttpOptions)    
+    update(settingsDict, defaultHttpOptions)
     #=========================================================================
 
     #=========================================================================
@@ -639,14 +638,14 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     siteDefaults = {
         'SITE_ID': settingsDict['SERVER_MODE'],
     }
-    
+
     if warns:
         warnSet('SITE_ID', settingsDict, siteDefaults)
 
     update(settingsDict, siteDefaults)
     populateEnvToSettings(list(siteDefaults.keys()), settingsDict, evalKeys=(
         'SITE_ID',
-    ))    
+    ))
     #=========================================================================
 
     #=========================================================================
@@ -664,13 +663,13 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
                 'django.contrib.messages.context_processors.messages',
             ],
             'debug': DEBUG,
-        },             
+        },
     }
-    
+
     templatesSettings = settingsDict.get('TEMPLATES', ())
     for p in templatesSettings:
         update(p, tSettings)
-    
+
     templatesDefaults = {
         'TEMPLATES' : [
             tSettings,
@@ -679,7 +678,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
 
     update(settingsDict, templatesDefaults)
     #=========================================================================
-    
+
     #=========================================================================
     #  AUTH PASSWORD_VALIDATORS
     authPasswordValidators = {
@@ -702,7 +701,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             },
         ]
     }
-    
+
     update(settingsDict, authPasswordValidators)
     #=========================================================================
 
@@ -721,10 +720,10 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         MIDDLEWARE = 'MIDDLEWARE'
     else:
         MIDDLEWARE = 'MIDDLEWARE_CLASSES'
-        
+
     if not DJANGO_VERSION >= 20000: # django < 2.0
         middleware.append('django.contrib.auth.middleware.SessionAuthenticationMiddleware')
-        
+
     pluginsDefaults = {
         'INSTALLED_APPS': (
             'django.contrib.admin',
@@ -741,7 +740,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         ),
         MIDDLEWARE: middleware,
     }
-    
+
     if warns:
         warnWithout('INSTALLED_APPS', settingsDict, pluginsDefaults)
 
@@ -774,7 +773,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     else:
         if module:
             import Iuno.cloud
-    
+
     iunoCloudDefaults = {
         'IUNO_CLOUD_ENABLE': False,
         'IUNO_CLOUD_SERVICES': None,
@@ -789,7 +788,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     }
 
     update(settingsDict, iunoCloudDefaults)
-    
+
     populateEnvToSettings((
         'IUNO_CLOUD_ENABLE',
         'IUNO_CLOUD_SERVICES',
@@ -827,22 +826,22 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         if warns:
             warnPackageDisable('Iuno.cloud', settingsDict, iunoCloudDefaults)
     #=========================================================================
-    
+
     #=========================================================================
-    # Iuno.deploy.   
+    # Iuno.deploy.
     try:
         checkImportPackage('Iuno.deploy', settingsDict)
     except ImportError:
         pass
     else:
         import Iuno.deploy
-    
+
     if settingsDict['SERVER_MODE'] != DEVELOPMENT:
         require('IUNO_SITE_DOMAIN_NAMES', settingsDict)
-    allowedHosts = Iuno.deploy.attachSettings(settingsDict, 
+    allowedHosts = Iuno.deploy.attachSettings(settingsDict,
         settingsDict.get('IUNO_SITE_DOMAIN_NAMES', None))
     #=========================================================================
- 
+
     #=========================================================================
     # Iuno.member.
     try:
@@ -851,7 +850,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         pass
     else:
         import Iuno.member
-        
+
         iunoMemberDefaults = {
             'IUNO_MEMBER_ENABLE': False,
             'IUNO_MEMBER_FACEBOOK_LOGIN_ENABLE': False,
@@ -861,8 +860,8 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         }
 
         update(settingsDict, iunoMemberDefaults)
-        
-        populateEnvToSettings(list(iunoMemberDefaults.keys()), settingsDict, 
+
+        populateEnvToSettings(list(iunoMemberDefaults.keys()), settingsDict,
                             evalKeys=list(iunoMemberDefaults.keys()))
 
         if settingsDict['IUNO_MEMBER_ENABLE']:
@@ -875,7 +874,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
                         )
         else: # IUNO_MEMBER_ENABLE = False
             if warns:
-                warnPackageDisable('Iuno.member', settingsDict, 
+                warnPackageDisable('Iuno.member', settingsDict,
                                    iunoMemberDefaults)
     #=========================================================================
 
@@ -887,15 +886,15 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         pass
     else:
         import Iuno.shop
-    
+
         iunoShopDefaults = {
             'IUNO_SHOP_ENABLE': False,
             'IUNO_SHOP_PAYPAL_ENABLE': False,
         }
 
         update(settingsDict, iunoShopDefaults)
-        
-        populateEnvToSettings(list(iunoShopDefaults.keys()), settingsDict, 
+
+        populateEnvToSettings(list(iunoShopDefaults.keys()), settingsDict,
                             evalKeys=list(iunoShopDefaults.keys()))
 
         if settingsDict['IUNO_SHOP_ENABLE']:
@@ -907,7 +906,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             if warns:
                 warnPackageDisable('Iuno.shop', settingsDict, iunoShopDefaults)
     #=========================================================================
-    
+
     #=========================================================================
     # Patch.
     from Iuno import Patch
@@ -920,7 +919,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
     #=========================================================================
 
     #=========================================================================
-    
+
     #=========================================================================
     # HAYSTACK
 
@@ -928,7 +927,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
         BASE_DIR = settingsDict['BASE_DIR']
     else:
         BASE_DIR = os.path.join(".")
-        
+
     WHOOSH = {
         'HAYSTACK_CONNECTIONS': {
             'default': {
@@ -938,7 +937,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             },
         },
     }
-    
+
     XAPIAN = {
         'HAYSTACK_CONNECTIONS': {
             'default': {
@@ -947,7 +946,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             },
         },
     }
-        
+
     if settingsDict['DEBUG'] or sys.platform.startswith("win32"):
         HAYSTACK_SETTINGS = WHOOSH
     else:
@@ -958,7 +957,7 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
             HAYSTACK_SETTINGS = WHOOSH
             warnings.warn(
                 "xapian import error, default use whoosh: %s" % str(e))
-    
+
     if not 'HAYSTACK_CONNECTIONS' in settingsDict:
         update(settingsDict, HAYSTACK_SETTINGS)
 
@@ -984,8 +983,8 @@ def attachSettings(settingsDict, warns=True, printDetail=True):
 # detect Server mode
 def detectServerMode(settingsDict):
     """
-    Detect the project's current server mode. 
-    
+    Detect the project's current server mode.
+
     @param settingsDict Settings dict.
     """
     mode = os.environ.get('SERVER_MODE', 'DEVELOPMENT')
@@ -996,8 +995,8 @@ def detectServerMode(settingsDict):
 
 def detectDebug(settingsDict):
     """
-    Detect the project's current debug mode. 
-    
+    Detect the project's current debug mode.
+
     @param settingsDict Settings dict.
     """
     _DEBUG = (True if settingsDict.get('SERVER_MODE', DEVELOPMENT) not in (
@@ -1005,3 +1004,123 @@ def detectDebug(settingsDict):
 
     return _DEBUG
 
+def checkImportPackageForAttachURLs(packageName):
+    """
+    Check the package you want to import and
+    raise an ImportError if the package is not available.
+
+    @param packageName Package name.
+    @param settingsDict Settings dict.
+    """
+    try:
+        return importlib.import_module(packageName)
+    except ImportError:
+        return False
+
+def attachDebugURLs(settings, urlpatterns):
+    """
+    Attach debug related urlpatterns.
+    It must be called in urls.py by passing urlpatterns.
+
+    @param settings Django project's settings module.
+    @param urlpatterns urlpatterns in urls.py.
+    """
+    from django.conf.urls import url, include
+
+    try:
+        import debug_toolbar
+    except:
+        pass
+    else:
+        if settings.DEBUG:
+            urlpatterns += patterns('',
+                url(r'^__debug__/', include(debug_toolbar.urls)),
+            )
+
+def attachURLs(settings, urlpatterns):
+    """
+    Attach patched (for example, debug and admin...etc.) urlpatterns.
+    It must be called in urls.py by passing urlpatterns.
+
+    @param settings Django project's settings module.
+    @param urlpatterns urlpatterns in urls.py.
+    """
+
+    from django.conf.urls import url, include
+    from des import urls as des_urls
+
+    urlpatterns += patterns('',
+        url(r'^django-des/', include(des_urls)),
+    )
+
+    #==========================================================================
+    # Iuno.member.
+    try:
+        module = checkImportPackageForAttachURLs('Iuno.member')
+    except ImportError:
+        pass
+    else:
+        if module:
+            import Iuno.member
+
+    if getattr(settings, 'IUNO_MEMBER_ENABLE', None) and module:
+        Iuno.member.attachURLs(settings, urlpatterns)
+
+    #=========================================================================
+    # Iuno.cloud.
+    try:
+        module = checkImportPackageForAttachURLs('Iuno.cloud')
+    except ImportError:
+        pass
+    else:
+        if module:
+            import Iuno.cloud
+
+    if getattr(settings, 'IUNO_CLOUD_ENABLE', None) and module:
+        attachCloudKws = {}
+        if hasattr(settings, 'IUNO_CLOUD_SERVICES'):
+            attachCloudKws['services'] = settings.IUNO_CLOUD_SERVICES
+
+        Iuno.cloud.attachURLs(settings, urlpatterns, **attachCloudKws)
+
+    #==========================================================================
+    # Iuno.shop.
+    try:
+        module = checkImportPackageForAttachURLs('Iuno.shop')
+    except ImportError:
+        pass
+    else:
+        if module:
+            import Iuno.shop
+
+    if getattr(settings, 'IUNO_SHOP_ENABLE', None) and module:
+        Iuno.shop.attachURLs(settings, urlpatterns)
+        
+    #==========================================================================
+    # Iuno.test.
+    addTestUrl = False
+    if hasattr(settings ,'IUNO_TEST_ENABLE'):
+        addTestUrl = settings.IUNO_TEST_ENABLE
+
+    if addTestUrl:
+        from Iuno.test.views import test
+        urlpatterns += patterns('',
+            url(r"^remote/test/$", test, name="remoteTest"),
+        )
+
+    if settings.DEBUG == False and settings.SERVER_MODE == settings.DEVELOPMENT:
+        if DJANGO_VERSION >= 11100: # django >= 1.11
+            from django.views.static import serve
+            urlpatterns += patterns('',
+                url(r'^media/(?P<path>.*)$', serve, {
+                    'document_root': settings.MEDIA_ROOT,
+                })
+            )
+        else:
+            urlpatterns += patterns('',
+                url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
+                    'document_root': settings.MEDIA_ROOT,
+                })
+            )
+
+
