@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 12447 2020-08-06 05:22:08Z Lavender $
+# $Id: __init__.py 12825 2021-03-31 13:28:58Z Lavender $
 #
 # Copyright (c) 2013 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2020-08-06 13:22:08 +0800 (Thu, 06 Aug 2020) $
-# $Revision: 12447 $
+# $Date: 2021-03-31 21:28:58 +0800 (週三, 31 三月 2021) $
+# $Revision: 12825 $
 
 import logging
 import traceback
@@ -27,9 +27,23 @@ from django.conf import settings
 from Iuppiter.Encoding import utf8, _unicode
 from Iuppiter.Json import serializer
 from Iuppiter.Logging import createLogger
-
+from Iuppiter.DjangoUtil import extendInstalledApps
+    
 logger = createLogger(__name__, 'Newsletter.log')
 
+def attachSettings(settingsLocals, backends, preferred, config):
+    """
+    Attach 'mail' related settings to settings.py.
+
+    @param settingsLocals locals() in settings.py.
+    @param backends Backend instances.
+    @param preferred Preferred backends.
+    @param config Configuration settings.
+    """
+    extendInstalledApps(settingsLocals, (
+        'Iuno.cloud.newsletter',
+    ))
+
 class EmailBackend(BaseEmailBackend):
 
     def __init__(self, host=None, port=None, username=None, password=None,
