@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 12485 2020-08-27 10:15:37Z Lavender $
+# $Id: __init__.py 12811 2021-03-24 12:13:54Z Casey $
 #
 # Copyright (c) 2015 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -11,18 +11,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Author: Lavender $ (last)
-# $Date: 2020-08-27 18:15:37 +0800 (週四, 27 八月 2020) $
-# $Revision: 12485 $
+# $Author: Casey $ (last)
+# $Date: 2021-03-24 20:13:54 +0800 (週三, 24 三月 2021) $
+# $Revision: 12811 $
 
 import os
 import sys
 import warnings
 
 from Iuno import warnWithout, update, require
-from Iuppiter.DjangoUtil import (extendInstalledApps, extendMiddlewareClasses, 
-                                 extendTemplateContextProcessors, 
-                                 extendAuthenticationBackend, DJANGO_VERSION)
+from Iuppiter.DjangoUtil import (extendInstalledApps, extendMiddlewareClasses,
+                                 extendTemplateContextProcessors,
+                                 extendAuthenticationBackend, DJANGO_VERSION,
+                                 patterns)
 
 
 def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
@@ -42,12 +43,12 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
 
     ADDING_INSTALLED_APPS = (
         'account',
-        'pinax_theme_bootstrap', 
+        'pinax_theme_bootstrap',
         'bootstrapform',
         'social_django',
         'django.contrib.sites',
     )
-    
+
     ADDING_MIDDLEWARE_CLASSES = (
         'account.middleware.LocaleMiddleware',
         'account.middleware.TimezoneMiddleware',
@@ -61,7 +62,7 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
     extendMiddlewareClasses(settingsDict, ADDING_MIDDLEWARE_CLASSES,
                             key='django.middleware.security.SecurityMiddleware')
     #=========================================================================
-    
+
     #=========================================================================
     # Email.
     emailDefaults = {
@@ -70,61 +71,61 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
 
     update(settingsDict, emailDefaults)
     #=========================================================================
-    
+
     #=========================================================================
     # authentication backends.
-    authenticationBackends = [            
+    authenticationBackends = [
         'account.auth_backends.EmailAuthenticationBackend',
         'django.contrib.auth.backends.ModelBackend',
         'social_core.backends.open_id.OpenIdAuth',
     ]
-    
+
     if enableFacebook:
         authenticationBackends.append('social_core.backends.facebook.FacebookOAuth2')
-        
+
     if enableGoogle:
         authenticationBackends.append('social_core.backends.google.GoogleOAuth2')
-            
+
     if enableYahoo:
         authenticationBackends.append('social_core.backends.yahoo.YahooOAuth2')
-        
+
     if enableLive:
         authenticationBackends.append('social_core.backends.live.LiveOAuth2')
 
     authenticationBackends = tuple(authenticationBackends)
-    
+
     extendAuthenticationBackend(settingsDict, authenticationBackends)
-    
+
     defaultSessionSerializers = {
-        'SESSION_SERIALIZER': 
+        'SESSION_SERIALIZER':
         'django.contrib.sessions.serializers.PickleSerializer'
     }
     update(settingsDict, defaultSessionSerializers)
     #=========================================================================
-    
+
     #=========================================================================
     #  Django templates.
     if DJANGO_VERSION >= 11000:
         contextProcessorsText = "django.template.context_processors.request"
     else:
         contextProcessorsText = "django.core.context_processors.request"
-        
+
     ADDING_TEMPLATES_CONTEXT_PROCESSORS = (
         "account.context_processors.account",
-        contextProcessorsText, 
+        contextProcessorsText,
         "pinax_theme_bootstrap.context_processors.theme",
         "social_django.context_processors.backends",
-        "social_django.context_processors.login_redirect",                             
+        "social_django.context_processors.login_redirect",
     )
-    
-    extendTemplateContextProcessors(settingsDict, 
+
+    extendTemplateContextProcessors(settingsDict,
                                     ADDING_TEMPLATES_CONTEXT_PROCESSORS)
     #=========================================================================
-    
+
     #=========================================================================
     #  social_auth pipeline.
     socialAuthPipelineDefaults = {
-        'SOCIAL_AUTH_PIPELINE' : ( 
+        'SOCIAL_AUTH_PIPELINE' : (
             'social_core.pipeline.social_auth.social_details',
             'social_core.pipeline.social_auth.social_uid',
             'social_core.pipeline.social_auth.social_user',
@@ -134,12 +135,12 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
             'social_core.pipeline.social_auth.associate_user',
             'social_core.pipeline.social_auth.load_extra_data',
             'social_core.pipeline.user.user_details',
-        ) 
+        )
     }
 
     update(settingsDict, socialAuthPipelineDefaults)
     #=========================================================================
-    
+
     #=========================================================================
     #  socail_auth basic settings.
     socailAuthSettingsDefaults = {
@@ -148,7 +149,7 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
         'SOCIAL_AUTH_NONCE_SERVER_URL_LENGTH' : 255,
         'SOCIAL_AUTH_ASSOCIATION_SERVER_URL_LENGTH' : 255,
     }
-    
+
     if enableFacebook:
         socailAuthSettingsDefaults[
             'SOCIAL_AUTH_FACEBOOK_PROFILE_EXTRA_PARAMS'] = {
@@ -158,15 +159,15 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
         socailAuthSettingsDefaults['SOCIAL_AUTH_FACEBOOK_SCOPE'] = ['email']
         require('SOCIAL_AUTH_FACEBOOK_KEY', settingsDict)
         require('SOCIAL_AUTH_FACEBOOK_SECRET', settingsDict)
-        
+
     if enableGoogle:
         require('SOCIAL_AUTH_GOOGLE_OAUTH2_KEY', settingsDict)
         require('SOCIAL_AUTH_GOOGLE_OAUTH2_SECRET', settingsDict)
-        
+
     if enableYahoo:
         require('SOCIAL_AUTH_YAHOO_OAUTH2_KEY', settingsDict)
         require('SOCIAL_AUTH_YAHOO_OAUTH2_SECRET', settingsDict)
-        
+
     if enableLive:
         socailAuthSettingsDefaults['SOCIAL_AUTH_LIVE_SCOPE'] = \
             ['wl.basic', 'wl.emails']
@@ -177,18 +178,18 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
     #=========================================================================
     # default login related value settings.
     # TODO: There are lots of default values yet to set, it's better to set them
-    # all. See 
+    # all. See
     # https://docs.djangoproject.com/en/1.11/ref/settings/#login-redirect-url
     # http://django-user-accounts.readthedocs.io/
-    # en/latest/settings.html#settings 
+    # en/latest/settings.html#settings
     # http://python-social-auth-docs.readthedocs.io/
     # en/latest/configuration/settings.html#urls-options
-    
+
     if 'LOGIN_URL' not in settingsDict:
         settingsDict['LOGIN_URL'] = '/accounts/login/'
     if 'LOGIN_REDIRECT_URL' not in settingsDict:
         settingsDict['LOGIN_REDIRECT_URL'] = '/accounts/profile/'
-    
+
     if 'SOCIAL_AUTH_LOGIN_URL' not in settingsDict:
         settingsDict['SOCIAL_AUTH_LOGIN_URL'] = settingsDict['LOGIN_URL']
     if 'SOCIAL_AUTH_LOGIN_REDIRECT_URL' not in settingsDict:
@@ -213,3 +214,15 @@ def attachSettings(settingsDict, enableFacebook=False, enableGoogle=False,
         https = os.environ.get('HTTPS', None)
         if https == 'on':
             settingsDict['DEFAULT_HTTP_PROTOCOL'] = 'https'
+
+def attachURLs(settings, urlpatterns):
+    from django.conf.urls import include, url
+
+    from .views import (
+        SignupView, LoginView, ConfirmEmailView, ChangePasswordView, LogoutView,
+        PasswordResetView, PasswordResetTokenView, SettingsView, DeleteView
+    )
+
+    urlpatterns += (
+        url(r'accounts/', include('Iuno.member.urls')),
+    )
