@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: __init__.py 12673 2021-01-06 05:40:40Z Lavender $
+# $Id: __init__.py 12785 2021-02-25 16:33:33Z Lavender $
 #
 # Copyright (c) 2018 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $ (last)
-# $Date: 2021-01-06 13:40:40 +0800 (Wed, 06 Jan 2021) $
-# $Revision: 12673 $
+# $Date: 2021-02-26 00:33:33 +0800 (週五, 26 二月 2021) $
+# $Revision: 12785 $
 
 import os
 import re
@@ -739,6 +739,11 @@ def attachURLs(settings, urlpatterns):
             
             # dashboard_promotions
             url(r'^shop/dashboard/content-blocks/', include(apps.get_app_config('dashboard_promotions').urls[0])),
+            
+            # invoice
+            url(r'^shop/orders/(?P<orderNumber>[\w-]*)/invoice/', 
+                views.InvoiceView.as_view(),
+                name='invoice'),
         )
     else:
         from oscar.app import application as shop
