@@ -20,8 +20,8 @@ from Iuno.test import RemoteTestCaseMixin
 
 class TestOscarDemo(TestCase, RemoteTestCaseMixin):
     
-    TEST_URL = "http://127.0.0.1:8000"
-    TEST_SETTINGS = "OscarDemo.TestSettings"
+    TEST_URL = "http://oscar.nuwainfo.com"
+    TEST_SETTINGS = "tests.TestSettings"
         
 
 if __name__ == "__main__":
